; -----------------------------------------------------------------------------------------------
; asc2bin16f                                                                           23/07/2020 
;
; Converte a binario 2 caracteres ASCII entre "0" (zero) e "F".
;
; Entrada: FSR aponta para o local da memoria RAM onde reside o primeiro caracter.
;
; Saida  : W contera o resultado da conversao: um numero binario entre 0 (zero) e 255.
;          STATUS<Z> = 1 .............. conversao ok
;          STATUS<Z> = 0 .............. conversao falhou (caracteres fora do intervalo permitido)
;
; -----------------------------------------------------------------------------------------------
asc2bin        ; Verifica o primeiro digito (MSB) de c1:c2=????.mmmm:????nnnn
                                                 ; FSR -> c1
               call    asc2bin_vdig              ; verifica limites do primeiro caracter (c1)
               btfss   STATUS, Z                 ; dentro dos limites???
               return                            ; NAO --> retorna com erro (Z=0)

               ; Verifica o segundo digito (LSB)
asc2bin1       incf    FSR, f                    ; FSR -> c2 (avanca para o segundo caracter)
               call    asc2bin_vdig              ; verifica limites do segundo caracter
               btfss   STATUS, Z                 ; dentro dos limites???
               return                            ; NAO --> retorna com erro (Z=0)
               
                                                 ; FSR -> c2 
asc2bin2       movlw   0x0F                      ; W = 0000.1111
               andwf   INDF, f                   ; c2= 0000.nnnn
               decf    FSR, f                    ; FSR -> c1 em c1:c2=????.mmmm:????.nnnn
               andwf   INDF, f                   ; c1= 0000.mmmm
               swapf   INDF, w                   ; W = mmmm.???? 
               incf    FSR, f                    ; FSR -> c2
               iorwf   INDF, w                   ; W = mmmm.nnnn
               bsf     STATUS, Z                 ; Z=1 --> ok
               return                            ; fim --> W=mmmm.nnnn

asc2bin_vdig   ; Verifica se o digito e' numerico
               movf    INDF,w                    ; pega o caracter  
               andlw   0xF0                      ; deixa so o MSB                 
               xorlw   0x30                      ; if(MSB == 0x30) -> numerico de 0 a 9 ???
               btfsc   STATUS, Z                 ; numerico de 0 a 9 ???
               goto    asc2bin09                 ; SIM --> testa o LSB do caracter
               ; Verifica se o digito e' letra maiuscula
               movf    INDF,w                    ; NAO --> pega o caracter  
               andlw   0xF0                      ; deixa so o MSB 
               xorlw   0x40                      ; if(MSB == 0x40) -> numerico de "A" a "F" ???
               btfsc   STATUS, Z                 ; numerico de "A" a "F" ???
               goto    asc2bin16                 ; SIM --> testa o LSB do caracter
               ; Verifica se o digito e' letra minuscula
               movf    INDF,w                    ; pega o caracter  
               andlw   0xF0                      ; deixa so o MSB 
               xorlw   0x60                      ; if(MSB == 0x60) -> numerico de "a" a "f" ???
               btfsc   STATUS, Z                 ; numerico de "a" a "f" ???
               goto    asc2bin16                 ; SIM --> testa o LSB do caracter
               goto    asc2binerro               ; NAO --> caracter fora dos limites
               ; Verifica limite do digito se for um numero
asc2bin09      movf    INDF,w                    ; pega o caracter
               andlw   0x0F                      ; deixa so o LSB
               sublw   .9                        ; W = 9 - W
               btfss   STATUS, C                 ; resultou negativo???
               goto    asc2binerro               ; NEGATIVO --> erro
               bsf     STATUS,Z                  ; POSITIVO --> Z=1
               return                            ; caracter dentro dos limites
               ; Verifica limite do digito se for uma letra de 'a' a 'f' ou de 'A' a 'F'
asc2bin16      decf    INDF,f                    ; ajusta
               movlw   0x0A
               addwf   INDF,f
               movf    INDF,w                    ; pega o caracter
               andlw   0x0F                      ; deixa so o LSB
               sublw   .15                       ; W = 15 - W
               btfss   STATUS, C                 ; resultou negativo???
               goto    asc2binerro               ; SIM --> erro
               bsf     STATUS,Z                  ; Z=1
               return                            ; caracter dentro dos limites
               ; Digito invalido
asc2binerro    bcf     STATUS,Z                  ; caracter fora dos limites
               retlw   .0                        ; erro


