; ---------------------------------------------------------------------------------------------
; INTERRUPCOES: Rotina de tratamento da interrupcao (Rx) da porta serial.            02/08/2020
;
; Esta rotina de interrupcao le uma linha de programa enviados pelo PC em formato binario.
; A leitura sera feita pelo tamanho enviado pelo primeiro byte da linha recebida.
; Os dados recebidos por esta rotina devem seguir o padrao da linha de programacao
; do arquivo .HEX:  BBAAaaTTDD.......DDCR
;
; O buffer de leitura apontado por FSR deve ser grande o suficiente para conter a mensagem e
; pode residir em qualquer banco da memoria RAM. O acesso ao buffer sera feito indiretamente 
; atraves do registrador FSR.
;
; O primeiro byte a ser recebido deve comtem o numero de bytes de dados (BB).
;
; Entrada: 1 - O registrador FSR deve apontar para o inicio de um buffer de leitura.
;          2 - SERFLGS<7> = 0
;
; Saida  : 1 - O bit SERFLAGS<SEREOT> sera ligado indicando que ha nova mensagem no buffer.
;          2 - O buffer de leitura estara preenchido com caracteres enviados pela
;              Porta Serial.
;
; ---------------------------------------------------------------------------------------------
FLIH           btfsc  PIR1, RCIF           ; interrupcao de porta serial???    
               goto   selectISR            ; SIM --> e' aqui mesmo!!!
               retfie                      ; NAO --> fim

               ; Selecao de funcao ISR
selectISR      btfsc  SERFLAGS, 7          ; ler binario???      
               goto   serPortISR           ; NAO --> ler ate o fim da linha (0x0D)
                                           ;         Esta rotina esta em 'serialport_services.inc'

serPortInt movwf   SERWR               ; salva W
           movf    FSR, w              ; pega o indexador
           movwf   SERFSR              ; salva indexador
           swapf   STATUS,w            ; obtem o STATUS
           movwf   SERST               ; salva o STATUS

           ; Pega o primeiro byte --> BB (dadaLen)
           movf    RCREG, w            ; le o primeiro byte (BB)
           movwf   INDF                ; armazena o tamanho no buffer
           addlw   .1                  ; + 1 para poder pegar o CRC
           movwf   SERSIZE             ; salva o tamanho
           incf    FSR, f              ; avanca apontador do buffer

           ; Le dataH, dataL e dataType
           movlw   .3                  ; ler + 3 bytes datH, dataL e dataType
           movwf   R4                  ; salva contador
norxerr2   bcf     PIR1, RCIF          ; limpa o flag de leitura
noexerr3   btfss   PIR1, RCIF          ; chegou um byte???
           goto    noexerr3            ; NAO --> espera chegar um byte
           movf    RCREG, w            ; pega o byte
           movwf   INDF                ; armazena o  byte no buffer
           incf    FSR, f              ; avanca apontador do buffer  
           decfsz  R4, f               ; leu 3 bytes???
           goto    norxerr2            ; NAO --> continua...  

          ; Le ate o fim dos dados, inclusive o CRC
lenxby2    bcf     PIR1, RCIF          ; limpa o flag de leitura
lenxby3    btfss   PIR1, RCIF          ; chegou um byte???
           goto    lenxby3             ; NAO --> espera chegar um byte
           movf    RCREG, w            ; le o dado
           movwf   INDF                ; armazena o dado no buffer
           incf    FSR, f              ; avanca apontador do buffer   
           decfsz  SERSIZE, f          ; fim dos dados???
           goto    lenxby2             ; NAO --> continua lendo

           ; Tenta ler o delimitador de fim da linha (0x0D) so para esgotar a leitura.
           ; Nada sera feito com isso.
w0x0d      bcf     PIR1, RCIF          ; limpa o flag de leitura
w0x0d1     btfss   PIR1, RCIF          ; chegou um byte???
           goto    w0x0d1              ; NAO --> espera chegar um byte
           movf    RCREG, w 
           movwf   INDF
           incf    FSR, f
           xorlw   0x0D
           btfss   STATUS, Z
           goto    w0x0d 
           movlw   0x0D
           movwf   INDF

           ; Restaura contexto
eostr2     movf    SERFSR, w           ; recupera FSR
           movwf   FSR                 ; restaura FSR
           swapf   SERST,w             ; recupera STATUS
           movwf   STATUS              ; restaura o STATUS antes da interrupcao
           swapf   SERWR,f             ; inverte nibles (LSB + MSB)
           swapf   SERWR,w             ; restaura W sem afetar o STATUS
           bsf     SERFLAGS, SEREOT    ; avisa que a leitura terminou
           retfie                      ; fim

