
  ifndef __FAMILIAS_SYMBOL
    #include "..\..\Includes\familias.inc"
    messg "LCDMACRO_Services - Arquivo familias.inc foi incluido"
  endif

  ifndef __LCDMACROS
#define __LCDMACROS  1

; --------------------------------------------------------------------------------------
; MACRO INSTRUCOES                                                            08/03/2017
;
; LCD_CONFIG lines, cols, tris, port, clock, data
; -----------------------------------------------
;      Inicializa RAM e monta a tabela de enderecos de linhas para LCD 2x16, 4x16 e 4x20.
;      SUPORTA FAMILIAS 16F e 18F.
;      Esta macro deve ser chamada na sessao DATA do codigo fonte.
;
;
;
;
; SET_CURSOR line, col, type
; --------------------------
;      Posiciona o cursor na linha e coluna especificadas.
;
;      Esta macro da suporte para as subrotinas de LCD Serial (lcdsr44780) e LCD Paralelo
;      (lcdhd44780).
;      Esta macro chama a funcao SR_SETCURSOR da subrotina "lcdsr44780.asm".
;
; CHAMADA:   SET_CURSOR    linha, coluna, tipo
;
; Onde:      linha ................ numero da linha do LCD
;            coluna ............... numero da coluna do LCD
;            tipo ................. "r" ou "R" linha e coluna estao em registradores
;                                   "i" ou "I" linha e coluna estao inline na macro
;
;      EXEMPLO:   SET_CURSOR  .2, .10, "i"
;
;
;
;
; SR_4MVC   string_address
; ------------------------
;      Esta macro pode ser utilizada com LCD serial (shift register) ou conectado normalmente 
;      em modo 4 bits.
;      EXEMPLO PARA A FAMILA 18F
;      -------------------------  
;      msg001    SR_4MVC  ms001
;      ms001     DB       "mensagem",0
;                .
;                .
;                call     msg001
;
;
;      EXEMPLO PARA A FAMILIA 16F
;      --------------------------
;      msg001    ADDWF  PCL,f
;                DT     "mensagem",0
;                .
;                .
;                call     msg001
;
;
;
; LCD_PRINT  RAM_address, length
; ------------------------------
;      Envia ao LCD 'len' caracteres da string localizada na posicao 'addr' da RAM.
;      Exemplo:    
;      mensagem    res    .12    ; contem 12 caracteres ASCII
;             
;                  LCD_PRINT   mensagem, .12 
;
;
;
;
; LCD_WRITE  "string"
; -------------------
;      LCD_WRITE - Envia a string inline na macro para o LCD.
;      Exemplo:
;                  LCD_WRITE  "Isto e uma mensagem" 
;
;
; --------------------------------------------------------------------------------------
;
  
  ifndef __LCDMACRO_SERVICES
#define __LCDMACRO_SERVICES  1       ; lcdmacro_services presente
  endif


; --------------------------------------------------------------------------------------
; Definicao de variaveis de controle que serao usadas pelas macros            07/03/2017
; LCD_LINES, SET_CURSOR e pela funcao SR_LINESET.
; --------------------------------------------------------------------------------------
  ifndef _lcdlines
          variable   _lcdlines
          variable   _lcdcols
          variable   _lcdsize
          variable   _lcdaddr1
          variable   _lcdaddr2
          variable   _lcdaddr3
          variable   _lcdaddr4
  endif

  ifndef _numq
         variable _numq
_numq    set   0
  endif

; ---------------------------------------------------------------------------------------------
; MACRO: LCD_CONFIG                                                                  08/03/2017
;
; Inicializa RAM e monta a tabela de enderecos de linhas para LCD 2x16, 4x16 e 4x20.
; SUPORTA FAMILIAS 16F e 18F.
; Esta macro deve ser chamada na sessao DATA do codigo fonte.
;
; EXEMPLO:
; Inicializa um LCD 2x16 no PORTA com pi no de clock em RA1 e pino data em RA0.
;
;         UDATA  0x20
;         LCD_CONFIG .2, .16, TRISA, PORTA, RA1, RA0
;  
;
; ---------------------------------------------------------------------------------------------
LCD_CONFIG   MACRO   lines, cols, trisx, portx, clockx, datax

; Geracao de registradores de controle na RAM e outros simbolos

SR_1         RES          1             ; registrador auxiliar 1
SR_2         RES          1             ; registrador auxiliar 2
SR_3         RES          1             ; registrador auxiliar 3
SR_4         RES          1             ; registrador auxiliar 4
SR_5         RES          1             ; registrador auxiliar 5
LCD_COMMAND  RES          1             ; buffer de comando e dados
SR_TIPO      RES          1             ; tipo de LCD
SR_CLK       equ          clockx        ; em TRISx este bit deve ser saida
SR_DAT       equ          datax         ; em TRISx este bit deve ser saida
SR_CHARS     equ          0             ; 0 - caracteres 5x7    1 - caracteres 5x10
#define      SR_TRIS      trisx         ; I/O
#define      SR_DATA      portx,SR_DAT  ; DATA
#define      SR_CLOCK     portx,SR_CLK  ; CLOCK


; Geracao da tabela de enderecos das linhas

_lcdlines set   lines
_lcdcols  set   cols
_lcdsize  set   _lcdlines * _lcdcols


  if _lcdsize==.32          ; LCD 2x16
_lcdaddr1 set  0x00         ; endereco da linha 1
_lcdaddr2 set  0x40         ; endereco da linha 2
_lcdaddr3 set  0x00         ; endereco da linha 3 - nao existe realmente
_lcdaddr4 set  0x40         ; endereco da linha 4 - nao existe realmente
  endif


  if _lcdsize==.64          ; LCD 4x16
_lcdaddr1 set  0x00         ; endereco da linha 1
_lcdaddr2 set  0x40         ; endereco da linha 2
_lcdaddr3 set  0x90         ; endereco da linha 3 
_lcdaddr4 set  0xD0         ; endereco da linha 4 
  endif

  if _lcdsize==.80          ; LCD 4x20
_lcdaddr1 set  0x00         ; endereco da linha 1
_lcdaddr2 set  0x40         ; endereco da linha 2
_lcdaddr3 set  0x14         ; endereco da linha 3 - nao existe realmente
_lcdaddr4 set  0x54         ; endereco da linha 4 - nao existe realmente
  endif

             ENDM



; -------------------------------------------------------------------------------------
; SET_CURSOR                                                      Spinelli - 25/01/2012
;
; Posiciona o cursor na linha e coluna especificadas.
;
; Esta macro da suporte para as subrotinas de LCD Serial (lcdsr44780) e LCD Paralelo
; (lcdhd44780).
;
; Esta macro chama a funcao SR_SETCURSOR da subrotina "lcdsr44780.asm".
;
; CHAMADA:   SET_CURSOR    linha, coluna, tipo
;
; Onde:      linha ................ numero da linha do LCD
;            coluna ............... numero da coluna do LCD
;            tipo ................. "r" ou "R" linha e coluna estao em registradores
;                                   "i" ou "I" linha e coluna estao inline na macro
;
; EXEMPLO:   SET_CURSOR  .2, .10, "i"
;
; -------------------------------------------------------------------------------------
SET_CURSOR   MACRO    line, col, type
   

   ; ---------------------------------------------------------------------------------- 
   ; Codigo para LCD paralelo - 24/07/2014
   ; ---------------------------------------------------------------------------------- 
   ifdef _PARALLEL_LCD
             IF  (type=="R") || (type=="r")
                 movf   line,w                   ; aqui, 'line' e' um registrador
                 movwf  LCD_R0
                 decf   LCD_R0,f
                 movf   col,w
                 movwf  LCD_R1
                 decf   LCD_R1,f
             ELSE
                 movlw  line                     ; aqui, 'line' e' uma constante
                 movwf  LCD_R0
                 decf   LCD_R0,f
                 movlw  col 
                 movwf  LCD_R1
                 decf   LCD_R1,f
             ENDIF
   
   ; ---------------------------------------------------------------------------------- 
   ; Codigo para LCD serial com Shift Register - 24/07/2014
   ; ---------------------------------------------------------------------------------- 
   else
             IF  (type=="R") || (type=="r")
                 movf   line,w                   ; aqui, 'line' e' um registrador
                 movwf  SR_1
                 decf   SR_1,f
                 movf   col,w
                 movwf  SR_2
                 decf   SR_2,f
             ELSE
                 movlw  line                     ; aqui, 'line' e' uma constante
                 movwf  SR_1
                 decf   SR_1,f
                 movlw  col
                 movwf  SR_2
                 decf   SR_2,f
             ENDIF

   endif 

                 call   SR_SETCURSOR

             ENDM  







; ---------------------------------------------------------------------------------------------
; SR_4MVC - move uma string para o LCD.
;
; Esta macro pode ser utilizada com LCD serial (shift register) ou conectado normalmente 
; em modo 4 bits.
;
;
; EXEMPLO PARA A FAMILA 18F
; -------------------------  
; msg001    SR_4MVC  ms001
; ms001     DB       "mensagem",0
;           .
;           .
;           call     msg001
;
;
; EXEMPLO PARA A FAMILIA 16F
; --------------------------
; msg001    ADDWF  PCL,f
;           DT     "mensagem",0
;           .
;           .
;           call     msg001
;  
; ---------------------------------------------------------------------------------------------
SR_4MVC      MACRO    str
_numq        set      _numq + 1
; ---------------------------------------------------------------------------------------------
;            CODIGO PARA PIC18F
; ---------------------------------------------------------------------------------------------
 IF __FAMILIA==18
             movlw   UPPER(str)
             movwf   TBLPTRU
             movlw   HIGH(str)
             movwf   TBLPTRH
             movlw   LOW(str)
             movwf   TBLPTRL
             call    SR_4MVC#v(_numq)
             return
       IFNDEF  SR_4MVC_INDICACOR
             variable  SR_4MVC_INDICACOR
       ENDIF
SR_4MVC#v(_numq) tblrd   *+
             movf    TABLAT,w
             xorlw   0x00
             btfsc   STATUS,Z
             return
             call    SR_SENDW
             goto    SR_4MVC#v(_numq)
     ENDIF

; ---------------------------------------------------------------------------------------------
;            CODIGO PARA PIC16
; ---------------------------------------------------------------------------------------------
 IF __FAMILIA<=16
   IFDEF _LCD_SERIAL
             clrf     LCD_R2                     ; 25/07/2014
S4#v(_numq)  movf     LCD_R2,w                   ; 25/07/2014
   ELSE
             clrf     SR_3                       ; 06/10/2013
S4#v(_numq)  movf     SR_3,w                     ; 06/10/2013
   ENDIF
             call     str
             xorlw    0x00
             btfsc    STATUS,Z
             return
             call     SR_SENDW                   ; 06/10/2013
   IFDEF _LCD_SERIAL
             incf     LCD_R2,f                   ; 25/07/2014
   ELSE
             incf     SR_3  ,f                   ; 06/10/2013
   ENDIF
             goto     S4#v(_numq)
  ENDIF
             ENDM



; ---------------------------------------------------------------------------------------------
; LCD_PRINT - Envia ao LCD 'len' caracteres da string localizada na                  06/03/2017
;             posicao 'addr' da RAM.
; 
; Exemplo:    
; mensagem    res    .12    ; contem 12 caracteres ASCII
;             
;             LCD_PRINT   mensagem, .12 
; 
; ---------------------------------------------------------------------------------------------
LCD_PRINT     MACRO   addr, len

  if __FAMILIA==16
              movlw   len
              movwf   SR_1
              movlw   addr
              movwf   FSR
              call    _lcd_print
  endif

  if __FAMILIA==18
              movlw   len
              movwf   SR_1
              lfsr    FSR0, addr
              call    _lcd_print
  endif

              ENDM




; ---------------------------------------------------------------------------------------------
; LCD_WRITE - Envia a string inline na macro para o LCD                              04/07/2015
;
; Exemplo:    LCD_WRITE  "Isto e uma mensagem" 
; 
; ---------------------------------------------------------------------------------------------
LCD_WRITE      MACRO   st 
 IF __FAMILIA==18
               call    lcd_sendstr
               dw      st,0x00
 ELSE
   ERROR "LCDMACRO_Services(Macro LCD_WRITE) - Familia nao suportada"
 ENDIF
               ENDM

  ENDIF    ; ifndef __LCDMACROS




; ------------------------------------------------------------------------
; LINK - Chama uma subrotina em qualquer pagina
;
; O registrador W sera salvo inicialmente em R13 pois pode se tratar
; de um parametro para a subrotina chamada.
;
; ESTA MACRO DESTROI O REGISTRADOR R13.
;
; ------------------------------------------------------------------------
LINK          MACRO    rtn
              movwf    R13                       ; salva o possivel parametro
              movlw    HIGH(rtn)                 ; endereco alto da subrotina
              movwf    PCLATH                    ; estabelece o long jump
              movf     R13,w                     ; recupera o possivel parametro
              call     rtn                       ; jump
              clrf     PCLATH                    ; volta a pagina zero
              ENDM



; -------------------------------------------------------------------------------------
; SETANY_CURSOR                                                  Spinelli - 31/09/2013
;
; Posiciona o cursor na linha e coluna especificadas.
; Esta macro chama a funcao SR_SETCURSOR da subrotina "lcdsr44780.asm" quando esta 
; reside alem da primeira pagina de ROM.
;
; CHAMADA:   SETANY_CURSOR    linha, coluna, tipo
;
; Onde:      linha ................ numero da linha do LCD
;            coluna ............... numero da coluna do LCD
;            tipo ................. "r" ou "R" linha e coluna estao em registradores
;                                   "i" ou "I" linha e coluna estao inline na macro
;
; EXEMPLO:   SET_CURSOR  .2, .10, "i"
;
; -------------------------------------------------------------------------------------
SETANY_CURSOR   MACRO    line,col,type
             IF  (type=="R") || (type=="r")
                 movf   line,w
                 movwf  SR_1
                 decf   SR_1,f
                 movf   col,w
                 movwf  SR_2
                 decf   SR_2,f
             ELSE
                 movlw  line 
                 movwf  SR_1
                 decf   SR_1,f
                 movlw  col
                 movwf  SR_2
                 decf   SR_2,f
             ENDIF
                 movlw  HIGH(SR_SETCURSOR)
                 movwf  PCLATH
                 call   SR_SETCURSOR
                 movlw  HIGH($)
                 movwf  PCLATH
             ENDM  


; --------------------------------------------------------------------
; SRANY_4MVC                                               20/09/2013
;
; MOVE, PARA O LCD, STRINGS QUE RESIDEM ALEM DA PRIMEIRA PAGINA DA ROM.
; Esta macro espera que a sobrotina "lcdsr44780.asm" tambem resida
; alem da primeira pagina.
;
; move uma string para o LCD quando a interface de 4 bits
; esta sendo utilizada juntamente com um shift register.
; "str" e o endereco de uma tabela que contem a string.
;
; Esta macro gera codigos diferentes para as familias 16F e 18F.
; Para a familia 16F, a string "str" deve ser definida com DT.
; Para a familia 18F, a string "str" deve ser definida com DB.
;
; --------------------------------------------------------------------
SRANY_4MVC      MACRO    str
_numq        set      _numq + 1
; ---------------------------------------------------------------------------------------------
;            CODIGO PARA PIC18F
; ---------------------------------------------------------------------------------------------
 IF __FAMILIA==18
             movlw   UPPER(str)
             movwf   TBLPTRU
             movlw   HIGH(str)
             movwf   TBLPTRH
             movlw   LOW(str)
             movwf   TBLPTRL
             movlw   HIGH(SR_4MVC#v(_numq))
             movwf   PCLATH
             call    SR_4MVC#v(_numq)
             movlw   HIGH($)
             movwf   PCLATH
             return
       IFNDEF  SR_4MVC_INDICACOR
             variable  SR_4MVC_INDICACOR
       ENDIF
SR_4MVC#v(_numq) tblrd   *+
             movf    TABLAT,w
             xorlw   0x00
             btfsc   STATUS,Z
             return
             movwf   LCD_COMMAND
             movlw   HIGH(SR_SENDCHR)
             movwf   PCLATH
             call    SR_SENDCHR
             movlw   HIGH($)
             movwf   PCLATH
             goto    SR_4MVC#v(_numq)
     ENDIF

; ---------------------------------------------------------------------------------------------
;            CODIGO PARA PIC16
; ---------------------------------------------------------------------------------------------
 IF __FAMILIA<=16
             clrf     R10
S4#v(_numq)  movlw    HIGH(str)
             movwf    PCLATH
             movf     R10,w
             call     str
             movwf    R11
             movlw    HIGH($)
             movwf    PCLATH
             movf     R11,w
             xorlw    0x00
             btfsc    STATUS,Z
             return
             movwf    LCD_COMMAND
             movlw    HIGH(SR_SENDCHR)
             movwf    PCLATH
             call     SR_SENDCHR
             movlw    HIGH($)
             movwf    PCLATH
             incf     R10,f
             goto     S4#v(_numq)
  ENDIF
             ENDM

