; ---------------------------------------------------------------
; Excitador da bomba multiplicadora de tensao          12/08/2020
;
; STATUS: FUNCIONA
;
; Este modulo gera um sinal PWM de 2KHZ no pino RB3 com periodo 
; de 512us (0,000512s) e Duty Cycle de aproximadamente 50% (256us).
; A funcao deste modulo e' ativar a bomba multiplicadora de
; tensao para gerar a tensao de programacao de 13V. 
;
; Fosc = 4MHZ
; Tosc = 1/Fosc ==> Tosc=1/4000000 ==> Tosc=250ns
;
; ---------------------------------------------------------------
; CALCULO DO VALOR DO REGISTRADOR PR2 PARA O PERIODO DO SINAL PWM
;
;           Periodo
; PR2 = ----------------  - 1
;       4*Tosc*TMR2Presc
;
; Seja o Periodo=512us --> 0,000512s
;
;           0,000512
; PR2 = ----------------  - 1   ===> PR2 = 32
;       4*0,000000250*16
;
; ---------------------------------------------------------------
; CALCULO DO VALOR DE CCPR1L:CCP1CON<5:4> PARA O DUTY CYCLE
;
;                           Duty Cycle
; (CCPR1L:CCP1CON<5:4>) = --------------
;                         Tosc*TMR2Presc
;
; Se quisermos o Duty Cycle a metade do periodo temos:
; Periodo=0,000512s  ==> DutyCycle=0,000256s 
;
;                           0,000256
; (CCPR1L:CCP1CON<5:4>) = -------------- 
;                         0,000000250*16
;
; (CCPR1L:CCP1CON<5:4>) = 64
;
; Sendo 64 = b'1000000' devemos carregar os dois bits LSB b'00' em CCP1CON<5:4>)
; e b'10000' (0x10 = 16) em CCPR1L.
;
; ---------------------------------------------------------------

pwmOn          clrf    CCP1CON
               clrf    T2CON
               clrf    TMR2
               movlw   b'00001100'     ; CCP1CON PWM MODE
               iorwf   CCP1CON, f      ; CCP1CON PWM MODE

               bsf     STATUS, RP0
               movlw   .32             ; valor do periodo
               movwf   PR2
               bcf     TRISB, 3        ; RB3 --> saida do PWM
               bcf     STATUS, RP0

               movlw   .16             ; valor MSB do Duty Cycle
               movwf   CCPR1L
               movlw   b'00000000'     ; valor LSB do Duty Cycle BITS <5:4>
               iorwf   CCP1CON, f

               movlw   b'00000010'      ; TMR2 Prescaler = 1:16
               iorwf   T2CON, f
               nop
               bsf     T2CON, TMR2ON    ; start TMR2
               return

pwmOff         clrf    CCP1CON
               clrf    T2CON
               clrf    TMR2
               return
