; -------------------------------------------------------------------------------------------
; PROGRAMADOR PIC                                                                  12/08/2020
; 
; STATUS: FUNCIONA
;
; PROGRAMADOR PIC PARA LINUX COM BOMBA DE TENSAO PWM.
;
; Este projeto fornece um programador PIC para o sistema operaciona  Linux.
;
; O circuito programador sera alimentado com a tensao de 5V fornecida por um conversor USB-Serial.
;
; O chip alvo devera ser retirado do circuito de aplicacao e colocado no programador para
; ser programado. Nao foi testado se funciona com ISCP.
;
; Para conseguir a tensao de programacao de 13V, o circuito programador usara um multiplicador
; de tensao controlado por um sinal PWM de 2KHz gerado no pino RB3 do programador PIC16F648A.
;
; NOTAS:
; 1 - Cuidado, a porta USB_Serial pode nao fornecer a corrente necessaria para operar o circuito
;     do programador. Nesse caso, use uma fonte externa de 5V para alimentar o circuito e retire
;     a alimentacao de 5V fornecida pela porta serial.
;
; -------------------------------------------------------------------------------------------
; MICROCONTROLADORES SUPORTADOS NESTA VERSAO                                       19/08/2020
;
;  8 PINOS - PIC12F675
;
; 14 PINOS - PIC16F688
;
; 18 PINOS - PIC16F627A, PIC16F628A, PIC16F648A
;
; -------------------------------------------------------------------------------------------


; -------------------------------------------------------------------------------------------
; HARDWARE
; Fosc ............. 4MHZ Interno
; RA0 .............. CLOCK (saida) ----------------------> ao pino RB6 do PIC alvo
; RA1 .............. DATA  (saida) ----------------------> ao pino RB7 do PIC alvo
; RA2 .............. Controle de alta tensao (13V) ------> ao transistor de controle do MCLR do PIC alvo (Active-Low)
; RA3 .............. Controle de alimentacao (5V)--------> ao transistor de controle do pino 14 do PIC alvo (Active-Low)
; RA5 .............. MCLR/RST/VPP 
; RB0 .............. Botao para debug -------------------> active low
; RB1/Rx ........... ao TX da serial do PC
; RB2/Tx ........... ao RX da serial do PC
; RB3 .............. PWM da bomba de 13V --> ao pino 2 c 74LS368A
; RB6 .............. LCD Data  ---> somente para debug (deve ser conectado com um resistor de 1K)
; RB7 .............. LCD Clock ---> somente para debug (deve ser conectado com um resistor de 1K)
;
; -------------------------------------------------------------------------------------------
	list      p=16F648A
	#include <p16F648A.inc>
	errorlevel  -302

#define  REC_RETRY     .10         ; numero de tentativas de regravacao
;#define  NO_CONFIG_BITS 1          ; nao gravar memoria de configuracao
#define  _VERIFY        1          ; executar a verificacao da gravacao de um byte  na memoria de programa
#define  _CHECK_SUM     1          ; verificar Check Sum
#define  LCD_REQUIRED   1          ; exibir mensagens no LCD
#define   TEST_EOT      1          ; verificar recebimento de EOT  


; -------------------------------------------------------------------------------------------------------
	__CONFIG   _CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_OFF & _INTOSC_OSC_NOCLKOUT 
; -------------------------------------------------------------------------------------------------------
;  
             #include     "Includes\familias.inc"
             #include     "Includes\_16ADD.mac"
             #include     "Includes\_16SUBTRACT.mac"
             #include     "Includes\serialport_services.inc"

  ifdef LCD_REQUIRED
             #include     "Includes\lcdmacro_services.inc"
  endif

; -------------------------------------------------------------------------
; MICROCONTROLADORES SUPORTADOS                                  19/08/2020
; -------------------------------------------------------------------------
; 8 PINOS
#define     PIC12F675   b'01111110'     ; 0x7E

; 14 PINOS
#define     PIC16F688   b'10001100'     ; 0x8C

; 18 PINOS
#define     PIC16F627A  b'10000010'     ; 0x82
#define     PIC16F628A  b'10000011'     ; 0x83
#define     PIC16F648A  b'10001000'     ; 0x88

; 28 PINOS

; 40 PINOS
; -------------------------------------------------------------------------



            UDATA        0x20
bkp1        res          1
bkp0        res          1
picretry    res          1

  ifdef LCD_REQUIRED
            LCD_CONFIG   .2, .16, TRISB, PORTB, .7, .6
  endif

DPMADDR     RES          2
DDMADDR     RES          2

cmd_pgm     RES          1
Scount      RES          1
Mcount      RES          1

      
; SERVICOS DE PORTA SERIAL
SERSIZE  res       .1
SERDLM   res       .1  ; delimitador de strings
SERWR    res       .1  ; Salvamento de contexto do registrador W
SERST    res       .1  ; Salvamento de contexto do registrador STATUS
SERFSR   res       .1  ; Salvamento de indexador
SERFLAGS res       .1  ; FLAGS DO SERVICO 
SEREOT   equ       .0  ;   1 --> fim de transmissao
;
serbuff  equ       0xA0 
pgrbuff  equ       0xD0
;
temp_temp   RES          1
delay_temp  RES          1
CLOCK       equ          .4000000
microchip   res          1           ; chip detectado
; -------------------------------------------------------------------------------------------
; pgmr.asm                                                                         24/03/2009
; REGISTRADORES DE TRABALHO PARA AS ROTINAS DO PROGRAMADOR DE PIC
; -------------------------------------------------------------------------------------------
pgmr_flags   RES     1      ; flags do servico de programacao
pgmrmode     equ     0      ; 0 -> prog mode Off       1 -> prog mode On
pgmrckvdd    equ     1      ; 0 ->  chk Vdd off        1 -> chk Vdd on
pgmrckvpp    equ     2      ; 0 ->  chk Vpp off        1 -> chk Vp on

;
pgmr_rg1     RES     1      ; registrador geral 1
pgmr_rg2     RES     1      ; registrador geral 1
pgmr_cmd     RES     1      ; command buffer
pgmr_data1   RES     1      ; MSB data buffer
pgmr_data0   RES     1      ; LSB data buffer
pgmr_bits    RES     1      ; contador de bits

; -------------------------------------------------------------------------------------------
; DEFINICAO DE SIMBOLOS PARA AS SUB-ROTINAS DE PROGRAMACAO
; -------------------------------------------------------------------------------------------
#define     PGMR_IO      TRISA    ; Registrador de controle das protas de I/O
#define     PGMR_DATA    PORTA    ; Porta de dados onde sera definido o bit de dados
#define     PGMR_CNTL    PORTA    ; Porta de controle onde sera definido o bit de clock
#define     PGMR_PWHI    PORTA    ; Porta onde serao definidos o bit de controle da alta voltagem (13V) - P_HI
#define     PGMR_PWLO    PORTA    ; Porta onde serao definidos o bit de controle da tensao de alimentacao (5V) - P_LO
P_CK        equ          0        ; bit de saida do clock
P_WD        equ          1        ; bit de saida de dado
P_RD        equ          1        ; bit de entrada de dado
P_HI        equ          2        ; bit de controle da alta voltagem
P_LO        equ          3        ; bit de controle da tensao de alimentacao
;
RESP_RDCM           equ    "1"    ; LER DADO NA MEMORIA DE CONFIGURACAO
RESP_RDDM           equ    "2"    ; LER DADO NA MEMORIA EEPROM
RESP_RDPM           equ    "3"    ; LER DADO NA MEMORIA DE PROGRAMA
RESP_ERASED         equ    "4"    ; APAGAR EEPROM
RESP_ERASEP         equ    "5"    ; APAGAR MEMORIA DE PROGRAMA
RESP_LDPM           equ    "6"    ; GRAVAR DADO NA MEMORIA DE PROGRAMA
RESP_CHKVDD         equ    "7"    ; VERIFICAR VDD (liga/desliga)
RESP_CHKVPP         equ    "8"    ; VERIFICAR VPP (liga/desliga)

; ---------------------------------------------------------------------------------------------------------------------
;                       COMANDOS DE PROGRAMACAO
; ---------------------------------------------------------------------------------------------------------------------
;                       CODIGO                    NOME                                DADOS
;                       -----------               --------------------------------    ---------------------------------   
LOAD_CFG   equ          b'00000000'             ; LOAD CONFIGURATION                  0,data(14),0
LOAD_DPM   equ          b'00000010'             ; LOAD DATA FOR PROGRAM MEMORY        0,data(14),0
LOAD_DDM   equ          b'00000011'             ; LOAD DATA FOR DATA MEMORY           0,data(8),zero(6),0
INC_ADDR   equ          b'00000110'             ; INCREMENT ADDRESS
READ_DPM   equ          b'00000100'             ; READ DATA FROM PROGRAM MEMORY       0,data(14),0
READ_DDM   equ          b'00000101'             ; READ DATA FROM DATA MEMORY          0,data(8),zero(6),0
BPOC       equ          b'00001000'             ; BEGIN PROGRAMMING ONLY CYCLE
BULK_EPM   equ          b'00001001'             ; BULK ERASE PROGRAM MEMORY
BULK_EDM   equ          b'00001011'             ; BULK ERASE DATA MEMORY

;
; Registradores usados na funcao 'chip_ldpm.inc'
crcalc      RES          .1
crrec       RES          .1
ptrBuff     RES          .1
ptrBytes    RES          .1

dataLen     RES          .1
dataH       RES          .1
dataL       RES          .1
dataType    RES          .1
dataBytes   RES          .18

; -------------------------------------------------------------------------------------------
; Registradores de trabalho                                                        22/07/2020
; Mantenha esses registradores na area comum da RAM (0x70).
; -------------------------------------------------------------------------------------------
R0      equ    0x70
R1      equ    0x71
R2      equ    0x72
R3      equ    0x73
R4      equ    0x74
R5      equ    0x75
R6      equ    0x76
R7      equ    0x77
R8      equ    0x78
R9      equ    0x79
R10     equ    0x7A
R11     equ    0x7B
R12     equ    0x7C
R13     equ    0x7D
R14     equ    0x7E
R15     equ    0x7F

; -------------------------------------------------------------------------------------------
RESET_VECTOR CODE       0x000
             goto       inicio
INT_VECTOR   CODE       0x004
             goto       FLIH
;
MAIN         CODE

; -------------------------------------------------------------------------------------------
; MENSAGENS DO PROGRAMA
; -------------------------------------------------------------------------------------------
  ifdef LCD_REQUIRED
msgOk        SR_4MVC    msok
msok         addwf      PCL,f
             dt         "OK",0

msgAddr      SR_4MVC    msaddr
msaddr       addwf      PCL,f
             dt         "ADDR=0x",0

msgChip      SR_4MVC    mschip
mschip       addwf      PCL,f
             dt         "CHIP=0x",0

msgEOJ       SR_4MVC    mseoj
mseoj        addwf      PCL,f
             dt         " EOJ",0

msgEOF       SR_4MVC    mseof
mseof        addwf      PCL,f
             dt         " EOF",0

msgData      SR_4MVC    msdata
msdata       addwf      PCL,f
             dt         "DATA=0x",0

msgDetect    SR_4MVC    msdetec
msdetec      addwf      PCL,f
             dt         "DETETANDO",0

msgCRC       SR_4MVC    mscrc
mscrc        addwf      PCL,f
             dt         " CRC=",0

msgErro      SR_4MVC    mserro
mserro       addwf      PCL,f
             dt         "ERRO: ",0

msgSkip      SR_4MVC    msskip
msskip       addwf      PCL,f
             dt         "CFG SKIP",0

msgRetry     SR_4MVC    msretry
msretry      addwf      PCL,f
             dt         "RETRY ",0
  endif



;
msACK        addwf      PCL, f
             dt         "ACK",0x0D
;
msEOT        addwf      PCL, f
             dt         "EOT",0x0D
;
msERC        addwf      PCL,f
             dt         "ERC",0x0D
;
msgSpin      addwf      PCL,f
             dt         "SPPP: ",0x0D
;
ms999        addwf      PCL,f
             dt         "ERROPT",0x0D
;
             #include   "Includes\delay_services.inc"

             SERIAL_FUNCTIONS 

; -------------------------------------------------------------------------------------------
inicio         movlw   0x07
               movwf   CMCON
; -------------------------------------------------------------------------------------------
; INICIALIZACAO DAS PORTAS DE I/O
; -------------------------------------------------------------------------------------------
               movlw   0x00
               movwf   PORTB
               movwf   PORTA
               bsf     STATUS,RP0
               clrf    TRISA
               movlw   b'00000110' 
               movwf   TRISB
               bcf     STATUS,RP0

reinicio       clrf    pgmr_flags
               call    picStop

   ifdef LCD_REQUIRED
               call    SR_4BITS
               call    SR_LCDCLEAR
               call    msgOk
   endif


; -------------------------------------------------------------------------------------------
;           INICIALIZACAO DA PORTA SERIAL
; -------------------------------------------------------------------------------------------
               SERIAL_OPEN .4, .9600, 0x0D

; -------------------------------------------------------------------------------------------
; Detecta o Microcontrolador                                                   18/07/2020
; Detecta o microcontrolador a ser programado e o identifica no registrador 'microchip'.
; Carrega os dados de cada um nos registradores DPMADDR e DDMADDR.
; O registrador 'microchip' deve conter um dos seguintes codigos:
; microchip = 0x82  --> 01.0000.010 --> PIC16F627A
;           = 0x83  --> 01.0000.011 --> PIC16F628A
;           = 0x88  --> 01.0001.000 --> PIC16F648A
;           = 0x00  --> nao identificado
; -------------------------------------------------------------------------------------------
               call    chip_detect           

; -------------------------------------------------------------------------------------------
; Rotina principal
; -------------------------------------------------------------------------------------------
redo           call    picStop                    ; desliga tensao de programacao
redo_nonstop   bsf     SERFLAGS, 7                ; ler ate o fim da linha (0x0D)
               SERIAL_READ serbuff                ; leitura da porta serial
               movlw   serbuff                    ; endereco do inicio do buffer
               movwf   FSR                        ; carrega indexador
               movf    INDF, w                    ; pega a resposta (1 byte)
               movwf   R15                        ; salva a resposta
; -------------------------------------------------------------------------------------------
;            VERIFICA COMANDO ENVIADO PELO PC
; -------------------------------------------------------------------------------------------
               movlw   RESP_ERASEP                ; verifica se deve limpar o chip         
               subwf   R15, w                     ;
               btfsc   STATUS,Z                   ; 
               goto    chip_clear                 ; vai limpar o chip
; -------------------------------------------------------------------------------------------
               movlw   RESP_LDPM                  ; verifica se deve gravar memoria de programa  
               subwf   R15, w                     ;
               btfsc   STATUS,Z                   ;  
               goto    chip_ldpm                  ; vai gravar memoria de programa  
; -------------------------------------------------------------------------------------------
               movlw   RESP_RDPM                  ; verifica se deve ler memoria de programa       
               subwf   R15, w                     ;
               btfsc   STATUS,Z                   ; 
               goto    chip_rdpm                  ; vai ler memoria de programa  
; -------------------------------------------------------------------------------------------
               movlw   RESP_RDDM                  ; verifica se deve ler memoria EEPROM        
               subwf   R15, w                     ;
               btfsc   STATUS,Z                   ; 
               goto    chip_rddm                  ; vai ler memoria EEPROM    
; -------------------------------------------------------------------------------------------
               movlw   RESP_RDCM                  ; verifica se deve ler memoria de configuracao    
               subwf   R15, w                     ;
               btfsc   STATUS,Z                   ;
               goto    chip_rdcm                  ; vai ler memoria de configuracao 
; -------------------------------------------------------------------------------------------
               movlw   RESP_ERASED                ; verifica se deve apagar EEPROM    
               subwf   R15, w                     ;
               btfsc   STATUS,Z                   ;
               goto    chip_cleardm               ; vai apagar EEPROM  
; -------------------------------------------------------------------------------------------
               movlw   RESP_CHKVDD                ; verifica se deve testar Vdd 
               subwf   R15, w                     ;
               btfsc   STATUS,Z                   ;
               goto    chipVddTest                ;  arquivo 'hardwaretst.inc"
; -------------------------------------------------------------------------------------------
               movlw   RESP_CHKVPP                ; verifica se deve testar Vpp 
               subwf   R15, w                     ; 
               btfsc   STATUS,Z                   ;
               goto    chipVppTest                ; arquivo 'hardwaretst.inc"
; -------------------------------------------------------------------------------------------

               SERIAL_WRITE msgSpin
               SERIAL_WRITE ms999
               goto    redo                       ; le o terminal

               #include "Includes\chip_detect.inc"         ; detetar PIC (subrotina)
               #include "Includes\chip_clear.inc"          ; apagar memoria de programa e config bits
               #include "Includes\chip_cleardm.inc"        ; apagar memoria EEPROM
               #include "Includes\chip_clearcm.inc"        ; apagar memoria de configuracao
               #include "Includes\chip_ldpm.inc"           ; gravar memoria de programacao
               #include "Includes\chip_rdpm.inc"           ; ler memoria de programa
               #include "Includes\chip_rddm.inc"           ; ler memoria EEPROM
               #include "Includes\chip_rdcm.inc"           ; ler config bits
               #include "Includes\chip_hardwaretst.inc"    ; testes de hardware (Vdd, Vpp, etc...)

; -------------------------------------------------------------------------------------------
; ROTINAS AUXILIARES
; -------------------------------------------------------------------------------------------
shexa          call    hextostr
               movf    R1, w
               call    serialPut
               movf    R0, w
               call    serialPut
               return
 ifdef LCD_REQUIRED
lcdHexa        call    hextostr
               movf    R1, w
               call    SR_SENDW
               movf    R0, w
               call    SR_SENDW
               return
  endif




; -------------------------------------------------------------------------------------------
  ifdef LCD_REQUIRED
              #include "Includes\lcdfunc_services.inc"
  endif
              #include "Includes\asc2bin16f.inc"
              #include "Includes\hextostr_services.inc"
              #include "Includes\comuns.inc"
              #include "Includes\picproglib.inc"
              #include "Includes\interrupts.inc"
              #include "Includes\confwrite.inc" 
              #include "Includes\pwm_p512us.inc"
; -------------------------------------------------------------------------------------------
;
EEProm         CODE    0x2100
x              DE      "SPPP - Programador de Porta Serial para Linux"
;
               END
